@@ -1,30 +1,31 @@
-# -*- coding: utf-8 -*-
 """Test initialization of prompt_toolkit shell"""
 
 import sys
+
+import pyte
 import pytest
 
 from xonsh.platform import minimum_required_ptk_version
+from xonsh.ptk_shell.shell import tokenize_ansi
+from xonsh.pytest.tools import ON_WINDOWS
+from xonsh.shell import Shell
 
 # verify error if ptk not installed or below min
 
-from xonsh.ptk_shell.shell import tokenize_ansi, remove_ansi_osc
-from xonsh.shell import Shell
-
 
 @pytest.mark.parametrize(
-    "ptk_ver, ini_shell_type, exp_shell_type, warn_snip, using_vended_ptk",
+    "ptk_ver, ini_shell_type, exp_shell_type, warn_snip",
     [
-        (None, "prompt_toolkit", "prompt_toolkit", None, True),
-        ((0, 5, 7), "prompt_toolkit", "prompt_toolkit", "is not supported", True),
-        ((1, 0, 0), "prompt_toolkit", "prompt_toolkit", "is not supported", True),
-        ((2, 0, 0), "prompt_toolkit", "prompt_toolkit", None, False),
-        ((2, 0, 0), "best", "prompt_toolkit", None, False),
-        ((2, 0, 0), "readline", "readline", None, False),
-        ((3, 0, 0), "prompt_toolkit", "prompt_toolkit", None, False),
-        ((3, 0, 0), "best", "prompt_toolkit", None, False),
-        ((3, 0, 0), "readline", "readline", None, False),
-        ((4, 0, 0), "prompt_toolkit", "prompt_toolkit", None, False),
+        (None, "prompt_toolkit", "readline", None),
+        ((0, 5, 7), "prompt_toolkit", "readline", "is not supported"),
+        ((1, 0, 0), "prompt_toolkit", "readline", "is not supported"),
+        ((2, 0, 0), "prompt_toolkit", "prompt_toolkit", None),
+        ((2, 0, 0), "best", "prompt_toolkit", None),
+        ((2, 0, 0), "readline", "readline", None),
+        ((3, 0, 0), "prompt_toolkit", "prompt_toolkit", None),
+        ((3, 0, 0), "best", "prompt_toolkit", None),
+        ((3, 0, 0), "readline", "readline", None),
+        ((4, 0, 0), "prompt_toolkit", "prompt_toolkit", None),
     ],
 )
 def test_prompt_toolkit_version_checks(
@@ -32,9 +33,8 @@ def test_prompt_toolkit_version_checks(
     ini_shell_type,
     exp_shell_type,
     warn_snip,
-    using_vended_ptk,
     monkeypatch,
-    xonsh_builtins,
+    xession,
 ):
 
     mocked_warn = ""
@@ -67,11 +67,7 @@ def test_prompt_toolkit_version_checks(
 
     act_shell_type = Shell.choose_shell_type(ini_shell_type, {})
 
-    if using_vended_ptk:
-        # ensure PTK has been unloaded and the vended version added to sys.path
-        assert len(old_syspath) < len(sys.path)
-    else:
-        assert len(old_syspath) == len(sys.path)
+    assert len(old_syspath) == len(sys.path)
 
     sys.path = old_syspath
 
@@ -106,30 +102,27 @@ def test_tokenize_ansi(prompt_tokens, ansi_string_parts):
 
 
 @pytest.mark.parametrize(
-    "raw_prompt, prompt, osc_tokens",
+    "line, exp",
     [
-        # no title
-        ("test prompt", "test prompt", []),
-        # starts w/ title
-        ("\033]0;TITLE THIS\007test prompt", "test prompt", ["\033]0;TITLE THIS\007"]),
-        # ends w/ title
-        ("test prompt\033]0;TITLE THIS\007", "test prompt", ["\033]0;TITLE THIS\007"]),
-        # title in the middle
-        ("test \033]0;TITLE THIS\007prompt", "test prompt", ["\033]0;TITLE THIS\007"]),
-        # title + iTerm2 OSC exapmle
-        (
-            "test \033]0;TITLE THIS\007prompt \033]133;A\007here",
-            "test prompt here",
-            ["\033]0;TITLE THIS\007", "\033]133;A\007"],
-        ),
+        [repr("hello"), None],
+        ["2 * 3", "6"],
     ],
 )
-def test_remove_ansi_osc(raw_prompt, prompt, osc_tokens):
-    checked_prompt, removed_osc = remove_ansi_osc(raw_prompt)
-    assert prompt == checked_prompt
-    assert len(removed_osc) == len(osc_tokens)
-    for removed, ref in zip(removed_osc, osc_tokens):
-        assert removed == ref
+@pytest.mark.xfail(
+    ON_WINDOWS,
+    reason="Recent versions use Proactor event loop. This may need some handling",
+)
+def test_ptk_prompt(line, exp, ptk_shell, capsys):
+    inp, out, shell = ptk_shell
+    inp.send_text(f"{line}\nexit\n")  # note: terminate with '\n'
+    shell.cmdloop()
+    screen = pyte.Screen(80, 24)
+    stream = pyte.Stream(screen)
 
+    out, _ = capsys.readouterr()
 
-# someday: initialize PromptToolkitShell and have it actually do something.
+    # this will remove render any color codes
+    stream.feed(out.strip())
+    out = screen.display[0].strip()
+
+    assert out.strip() == (exp or line)
