@@ -1,39 +1,41 @@
-# -*- coding: utf-8 -*-
 """Lexer for xonsh code.
 
 Written using a hybrid of ``tokenize`` and PLY.
 """
 import io
-import re
 
 # 'keyword' interferes with ast.keyword
 import keyword as kwmod
-
-from xonsh.ply.ply.lex import LexToken
+import re
+import typing as tp
 
 from xonsh.lazyasd import lazyobject
 from xonsh.platform import PYTHON_VERSION_INFO
+from xonsh.ply.ply.lex import LexToken
 from xonsh.tokenize import (
-    OP,
-    IOREDIRECT,
-    STRING,
-    DOLLARNAME,
-    NUMBER,
-    SEARCHPATH,
-    NEWLINE,
-    INDENT,
-    DEDENT,
-    NL,
+    CASE,
     COMMENT,
+    DEDENT,
+    DOLLARNAME,
     ENCODING,
     ENDMARKER,
-    NAME,
     ERRORTOKEN,
     GREATER,
+    HAS_WALRUS,
+    INDENT,
+    IOREDIRECT,
     LESS,
+    MATCH,
+    NAME,
+    NEWLINE,
+    NL,
+    NUMBER,
+    OP,
     RIGHTSHIFT,
-    tokenize,
+    SEARCHPATH,
+    STRING,
     TokenError,
+    tokenize,
 )
 
 
@@ -111,6 +113,11 @@ def token_map():
 
         tm[ASYNC] = "ASYNC"
         tm[AWAIT] = "AWAIT"
+    if HAS_WALRUS:
+        tm[(OP, ":=")] = "COLONEQUAL"
+    # python 3.10 (backwards and name token compatible) tokens
+    tm[MATCH] = "MATCH"
+    tm[CASE] = "CASE"
     return tm
 
 
@@ -134,7 +141,7 @@ def handle_name(state, token):
     if state["pymode"][-1][0]:
         if needs_whitespace and not has_whitespace:
             pass
-        elif token.string in kwmod.kwlist:
+        elif token.string in kwmod.kwlist + ["match", "case"]:
             typ = token.string.upper()
         yield _new_token(typ, token.string, token.start)
     else:
@@ -156,7 +163,7 @@ def _end_delimiter(state, token):
             e = '"{}" at {} ends "{}" at {} (expected "{}")'
             return e.format(s, (l, c), orig, pos, match)
     else:
-        return 'Unmatched "{}" at line {}, column {}'.format(s, l, c)
+        return f'Unmatched "{s}" at line {l}, column {c}'
 
 
 def handle_rparen(state, token):
@@ -164,7 +171,7 @@ def handle_rparen(state, token):
     Function for handling ``)``
     """
     e = _end_delimiter(state, token)
-    if e is None:
+    if e is None or state["tolerant"]:
         state["last"] = token
         yield _new_token("RPAREN", ")", token.start)
     else:
@@ -174,7 +181,7 @@ def handle_rparen(state, token):
 def handle_rbrace(state, token):
     """Function for handling ``}``"""
     e = _end_delimiter(state, token)
-    if e is None:
+    if e is None or state["tolerant"]:
         state["last"] = token
         yield _new_token("RBRACE", "}", token.start)
     else:
@@ -186,7 +193,7 @@ def handle_rbracket(state, token):
     Function for handling ``]``
     """
     e = _end_delimiter(state, token)
-    if e is None:
+    if e is None or state["tolerant"]:
         state["last"] = token
         yield _new_token("RBRACKET", "]", token.start)
     else:
@@ -330,12 +337,11 @@ def handle_token(state, token):
 
     Parameters
     ----------
-
-    state :
+    state
         The current state of the lexer, including information about whether
         we are in Python mode or subprocess mode, which changes the lexer's
         behavior.  Also includes the stream of tokens yet to be considered.
-    token :
+    token
         The token (from ``tokenize``) currently under consideration
     """
     typ = token.type
@@ -358,11 +364,11 @@ def handle_token(state, token):
         state["last"] = token
         yield _new_token(token_map[typ], st, token.start)
     else:
-        m = "Unexpected token: {0}".format(token)
+        m = f"Unexpected token: {token}"
         yield _new_token("ERRORTOKEN", m, token.start)
 
 
-def get_tokens(s):
+def get_tokens(s, tolerant):
     """
     Given a string containing xonsh code, generates a stream of relevant PLY
     tokens using ``handle_token``.
@@ -371,14 +377,15 @@ def get_tokens(s):
         "indents": [0],
         "last": None,
         "pymode": [(True, "", "", (0, 0))],
-        "stream": tokenize(io.BytesIO(s.encode("utf-8")).readline),
+        "stream": tokenize(io.BytesIO(s.encode("utf-8")).readline, tolerant),
+        "tolerant": tolerant,
     }
     while True:
         try:
             token = next(state["stream"])
             yield from handle_token(state, token)
         except StopIteration:
-            if len(state["pymode"]) > 1:
+            if len(state["pymode"]) > 1 and not tolerant:
                 pm, o, m, p = state["pymode"][-1]
                 l, c = p
                 e = 'Unmatched "{}" at line {}, column {}'
@@ -404,12 +411,12 @@ def _new_token(type, value, pos):
     return o
 
 
-class Lexer(object):
+class Lexer:
     """Implements a lexer for the xonsh language."""
 
-    _tokens = None
+    _tokens: tp.Optional[tp.Tuple[str, ...]] = None
 
-    def __init__(self):
+    def __init__(self, tolerant=False):
         """
         Attributes
         ----------
@@ -419,27 +426,37 @@ class Lexer(object):
             The last token seen.
         lineno : int
             The last line number seen.
+        tolerant : bool
+            Tokenize without extra checks (e.g. paren matching).
+            When True, ERRORTOKEN contains the erroneous string instead of an error msg.
 
         """
         self.fname = ""
         self.last = None
         self.beforelast = None
+        self._tolerant = tolerant
+        self._token_stream = iter(())
+
+    @property
+    def tolerant(self):
+        return self._tolerant
 
     def build(self, **kwargs):
         """Part of the PLY lexer API."""
         pass
 
     def reset(self):
-        pass
+        self._token_stream = iter(())
+        self.last = None
+        self.beforelast = None
 
     def input(self, s):
         """Calls the lexer on the string s."""
-        self.token_stream = get_tokens(s)
+        self._token_stream = get_tokens(s, self._tolerant)
 
     def token(self):
         """Retrieves the next token."""
-        self.beforelast = self.last
-        self.last = next(self.token_stream, None)
+        self.beforelast, self.last = self.last, next(self._token_stream, None)
         return self.last
 
     def __iter__(self):
@@ -452,28 +469,28 @@ class Lexer(object):
         """Splits a string into a list of strings which are whitespace-separated
         tokens.
         """
-        vals = []
         self.input(s)
+        elements = []
         l = c = -1
         ws = "WS"
         nl = "\n"
-        for t in self:
-            if t.type == ws:
+        for token in self:
+            if token.type == ws:
                 continue
-            elif l < t.lineno:
-                vals.append(t.value)
-            elif len(vals) > 0 and c == t.lexpos:
-                vals[-1] = vals[-1] + t.value
+            elif l < token.lineno:
+                elements.append(token.value)
+            elif len(elements) > 0 and c == token.lexpos:
+                elements[-1] = elements[-1] + token.value
             else:
-                vals.append(t.value)
-            nnl = t.value.count(nl)
+                elements.append(token.value)
+            nnl = token.value.count(nl)
             if nnl == 0:
-                l = t.lineno
-                c = t.lexpos + len(t.value)
+                l = token.lineno
+                c = token.lexpos + len(token.value)
             else:
-                l = t.lineno + nnl
-                c = len(t.value.rpartition(nl)[-1])
-        return vals
+                l = token.lineno + nnl
+                c = len(token.value.rpartition(nl)[-1])
+        return elements
 
     #
     # All the tokens recognized by the lexer
@@ -481,6 +498,9 @@ class Lexer(object):
     @property
     def tokens(self):
         if self._tokens is None:
+            kwlist = kwmod.kwlist[:]
+            if (3, 9, 0) <= PYTHON_VERSION_INFO < (3, 10):
+                kwlist.remove("__peg_parser__")
             t = (
                 tuple(token_map.values())
                 + (
@@ -502,7 +522,7 @@ class Lexer(object):
                     "ATDOLLAR_LPAREN",  # @$(
                     "ERRORTOKEN",  # whoops!
                 )
-                + tuple(i.upper() for i in kwmod.kwlist)
+                + tuple(i.upper() for i in kwlist)
             )
             self._tokens = t
         return self._tokens
