@@ -1,33 +1,32 @@
 """Subprocess specification and related utilities."""
-import os
+import contextlib
+import inspect
 import io
+import os
+import pathlib
 import re
-import sys
-import stat
 import shlex
 import signal
-import inspect
-import pathlib
+import stat
 import subprocess
-import contextlib
+import sys
 
-from xonsh.built_ins import XSH
-import xonsh.tools as xt
-import xonsh.lazyasd as xl
-import xonsh.platform as xp
 import xonsh.environ as xenv
-import xonsh.lazyimps as xli
 import xonsh.jobs as xj
-
-from xonsh.procs.readers import ConsoleParallelReader
-from xonsh.procs.posix import PopenThread
-from xonsh.procs.proxies import ProcProxy, ProcProxyThread
+import xonsh.lazyasd as xl
+import xonsh.lazyimps as xli
+import xonsh.platform as xp
+import xonsh.tools as xt
+from xonsh.built_ins import XSH
 from xonsh.procs.pipelines import (
-    pause_call_resume,
+    STDOUT_CAPTURE_KINDS,
     CommandPipeline,
     HiddenCommandPipeline,
-    STDOUT_CAPTURE_KINDS,
+    pause_call_resume,
 )
+from xonsh.procs.posix import PopenThread
+from xonsh.procs.proxies import ProcProxy, ProcProxyThread
+from xonsh.procs.readers import ConsoleParallelReader
 
 
 @xl.lazyobject
@@ -164,16 +163,12 @@ def _REDIR_OUT():
 
 @xl.lazyobject
 def _E2O_MAP():
-    return frozenset(
-        {"{}>{}".format(e, o) for e in _REDIR_ERR for o in _REDIR_OUT if o != ""}
-    )
+    return frozenset({f"{e}>{o}" for e in _REDIR_ERR for o in _REDIR_OUT if o != ""})
 
 
 @xl.lazyobject
 def _O2E_MAP():
-    return frozenset(
-        {"{}>{}".format(o, e) for e in _REDIR_ERR for o in _REDIR_OUT if o != ""}
-    )
+    return frozenset({f"{o}>{e}" for e in _REDIR_ERR for o in _REDIR_OUT if o != ""})
 
 
 def _is_redirect(x):
@@ -184,7 +179,7 @@ def safe_open(fname, mode, buffering=-1):
     """Safely attempts to open a file in for xonsh subprocs."""
     # file descriptors
     try:
-        return io.open(fname, mode, buffering=buffering)
+        return open(fname, mode, buffering=buffering)
     except PermissionError:
         raise xt.XonshError(f"xonsh: {fname}: permission denied")
     except FileNotFoundError:
@@ -486,7 +481,7 @@ class SubprocSpec:
                     return self.cls(
                         ["man", cmd0.rstrip("?")], bufsize=bufsize, **kwargs
                     )
-            e = "xonsh: subprocess mode: command not found: {0}".format(cmd0)
+            e = f"xonsh: subprocess mode: command not found: {cmd0}"
             env = XSH.env
             sug = xt.suggest_commands(cmd0, env)
             if len(sug.strip()) > 0:
@@ -710,7 +705,7 @@ class SubprocSpec:
 
 
 def _safe_pipe_properties(fd, use_tty=False):
-    """Makes sure that a pipe file descriptor properties are sane."""
+    """Makes sure that a pipe file descriptor properties are reasonable."""
     if not use_tty:
         return
     # due to some weird, long standing issue in Python, PTYs come out
