@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """The prompt_toolkit based xonsh shell."""
 import os
 import re
@@ -6,36 +5,36 @@ import sys
 from functools import wraps
 from types import MethodType
 
-from xonsh.built_ins import XSH
-from xonsh.events import events
-from xonsh.base_shell import BaseShell
-from xonsh.ptk_shell.formatter import PTKPromptFormatter
-from xonsh.shell import transform_command
-from xonsh.tools import print_exception, print_warning, carriage_return
-from xonsh.platform import HAS_PYGMENTS, ON_WINDOWS, ON_POSIX
-from xonsh.style_tools import partial_color_tokenize, _TokenType, DEFAULT_STYLE_DICT
-from xonsh.lazyimps import pygments, pyghooks, winutils
-from xonsh.pygments_cache import get_all_styles
-from xonsh.ptk_shell.history import PromptToolkitHistory, _cust_history_matches
-from xonsh.ptk_shell.completer import PromptToolkitCompleter
-from xonsh.ptk_shell.key_bindings import load_xonsh_bindings
-
 from prompt_toolkit import ANSI
 from prompt_toolkit.auto_suggest import AutoSuggestFromHistory
 from prompt_toolkit.enums import EditingMode
+from prompt_toolkit.formatted_text import PygmentsTokens, to_formatted_text
+from prompt_toolkit.history import ThreadedHistory
 from prompt_toolkit.key_binding.bindings.emacs import (
     load_emacs_shift_selection_bindings,
 )
-from prompt_toolkit.key_binding.key_bindings import merge_key_bindings
 from prompt_toolkit.key_binding.bindings.named_commands import get_by_name
-from prompt_toolkit.history import ThreadedHistory
-from prompt_toolkit.shortcuts import print_formatted_text as ptk_print
+from prompt_toolkit.key_binding.key_bindings import merge_key_bindings
 from prompt_toolkit.shortcuts import CompleteStyle
+from prompt_toolkit.shortcuts import print_formatted_text as ptk_print
 from prompt_toolkit.shortcuts.prompt import PromptSession
-from prompt_toolkit.formatted_text import PygmentsTokens, to_formatted_text
-from prompt_toolkit.styles import merge_styles, Style
+from prompt_toolkit.styles import Style, merge_styles
 from prompt_toolkit.styles.pygments import pygments_token_to_classname
 
+from xonsh.base_shell import BaseShell
+from xonsh.built_ins import XSH
+from xonsh.events import events
+from xonsh.lazyimps import pyghooks, pygments, winutils
+from xonsh.platform import HAS_PYGMENTS, ON_POSIX, ON_WINDOWS
+from xonsh.ptk_shell.completer import PromptToolkitCompleter
+from xonsh.ptk_shell.formatter import PTKPromptFormatter
+from xonsh.ptk_shell.history import PromptToolkitHistory, _cust_history_matches
+from xonsh.ptk_shell.key_bindings import load_xonsh_bindings
+from xonsh.pygments_cache import get_all_styles
+from xonsh.shell import transform_command
+from xonsh.style_tools import DEFAULT_STYLE_DICT, _TokenType, partial_color_tokenize
+from xonsh.tools import carriage_return, print_exception, print_warning
+
 try:
     from prompt_toolkit.clipboard import DummyClipboard
     from prompt_toolkit.clipboard.pyperclip import PyperclipClipboard
@@ -44,7 +43,13 @@ try:
 except ImportError:
     HAVE_SYS_CLIPBOARD = False
 
-ANSI_OSC_PATTERN = re.compile("\x1b].*?\007")
+try:
+    from prompt_toolkit.cursor_shapes import ModalCursorShapeConfig
+
+    HAVE_CURSOR_SHAPE = True
+except ImportError:
+    HAVE_CURSOR_SHAPE = False
+
 CAPITAL_PATTERN = re.compile(r"([a-z])([A-Z])")
 Token = _TokenType()
 
@@ -85,19 +90,6 @@ def tokenize_ansi(tokens):
     return ansi_tokens
 
 
-def remove_ansi_osc(prompt):
-    """Removes the ANSI OSC escape codes - ``prompt_toolkit`` does not support them.
-    Some terminal emulators - like iTerm2 - uses them for various things.
-
-    See: https://www.iterm2.com/documentation-escape-codes.html
-    """
-
-    osc_tokens = ANSI_OSC_PATTERN.findall(prompt)
-    prompt = ANSI_OSC_PATTERN.sub("", prompt)
-
-    return prompt, osc_tokens
-
-
 def _pygments_token_to_classname(token):
     """Converts pygments Tokens, token names (strings) to PTK style names."""
     if token and isinstance(token, str):
@@ -309,6 +301,10 @@ class PromptToolkitShell(BaseShell):
         completer = None if completions_display == "none" else self.pt_completer
 
         events.on_timingprobe.fire(name="on_pre_prompt_tokenize")
+
+        # clear prompt level cache
+        env["PROMPT_FIELDS"].reset()
+
         get_bottom_toolbar_tokens = self.bottom_toolbar_tokens
         if env.get("UPDATE_PROMPT_ON_KEYPRESS"):
             get_prompt_tokens = self.prompt_tokens
@@ -359,7 +355,6 @@ class PromptToolkitShell(BaseShell):
             "refresh_interval": refresh_interval,
             "complete_in_thread": complete_in_thread,
         }
-
         if env["ENABLE_ASYNC_PROMPT"]:
             # once the prompt is done, update it in background as each future is completed
             prompt_args["pre_run"] = self.prompt_formatter.start_update
@@ -367,6 +362,8 @@ class PromptToolkitShell(BaseShell):
             for attr, val in self.get_lazy_ptk_kwargs():
                 prompt_args[attr] = val
 
+        if editing_mode == EditingMode.VI and HAVE_CURSOR_SHAPE:
+            prompt_args["cursor"] = ModalCursorShapeConfig()
         events.on_pre_prompt.fire()
         line = self.prompter.prompt(**prompt_args)
         events.on_post_prompt.fire()
@@ -428,16 +425,6 @@ class PromptToolkitShell(BaseShell):
         except Exception:  # pylint: disable=broad-except
             print_exception()
 
-        p, osc_tokens = remove_ansi_osc(p)
-
-        if kwargs.get("handle_osc_tokens"):
-            # handle OSC tokens
-            for osc in osc_tokens:
-                if osc[2:4] == "0;":
-                    env["TITLE"] = osc[4:-1]
-                else:
-                    print(osc, file=sys.__stdout__, flush=True)
-
         toks = partial_color_tokenize(p)
 
         return tokenize_ansi(PygmentsTokens(toks))
@@ -448,7 +435,7 @@ class PromptToolkitShell(BaseShell):
             carriage_return()
             self._first_prompt = False
 
-        tokens = self._get_prompt_tokens("PROMPT", "message", handle_osc_tokens=True)
+        tokens = self._get_prompt_tokens("PROMPT", "message")
         self.settitle()
         return tokens
 
@@ -474,7 +461,7 @@ class PromptToolkitShell(BaseShell):
         """Displays dots in multiline prompt"""
         if is_soft_wrap:
             return ""
-        width = width - 1
+        width -= 1
         dots = XSH.env.get("MULTILINE_PROMPT")
         dots = dots() if callable(dots) else dots
         if not dots:
